@@ -1,13 +1,13 @@
 # -----------------------------
 # -- kongodb --
-# database module
+# database
 # -----------------------------
 
 
 import copy 
 from urllib.parse import urlparse
 from typing import Any, List, Union
-from . import adapters, dictquery, lib, cursor, types, operators, exceptions
+from . import adapters, dict_mutator, dict_query, lib, types, exceptions
 
 
 SQL_DATETIME_FORMAT = "YYYY-MM-DD HH:mm:ss"
@@ -77,7 +77,7 @@ class __Item(dict):
         Returns:    
             int - the value that was incremented
         """
-        op = "%s:$incr" % self._make_path(path)
+        op = "%s:$incr" % self._make_path(path)        
         oplog = self._update({op: incr})
         return oplog.get(op)
 
@@ -313,11 +313,17 @@ class Database(object):
 
 class CollectionItem(__Item):
     """
-    CollectionItem/ActiveRecord
+    CollectionItem
 
     Every row is a document in Kongodb
     """
-    SUBCOLLECTION_NAMESPACE = "/subcollection/%s"
+
+    # item _id
+    _id = None
+
+    # items subcollections
+    _subcollections = {}
+    
 
     def __init__(self, collection, row: dict):
         self._collection = collection
@@ -343,24 +349,36 @@ class CollectionItem(__Item):
         self._clear_self()
 
     def subcollection(self, name: str, constraints: list = None):
+        """
+        Select a subcollection
+        """
+
+        # ensuring constraints
         collection_info = _CollectionInfo__(self._collection, name)
         _constraints = collection_info.get_subcollection_constraints(name) or []
         if constraints and set(constraints) != set(_constraints):
             collection_info.set_subcollection_constraints(name, constraints)
             _constraints = constraints
+
         return SubCollection(item=self, name=name, constraints=_constraints)
 
     @property
-    def subcollections(self):
-        return list(self.get("/collections").keys()) or []
+    def subcollections(self) -> list:
+        return list(self._subcollections.keys()) or []
 
     def drop_subcollection(self, name: str):
         try:
-            self.unset("/collections.%s" % name)
+            if name in self._subcollections:
+                del self._subcollections[name]
+            self.set("/subcollections", self._subcollections)
         except KeyError as _:
             pass
         return True
 
+    def _set_subcollection(self, name:str, data:Any):
+        self._subcollections[name] = data
+        self.set("/subcollections", self._subcollections)
+
     def save(self):
         """
         To commit the data when it's mutated outside.
@@ -371,14 +389,13 @@ class CollectionItem(__Item):
         data = dict(self)
         self._update(data)
 
-    def _update(self, ops: dict):
+    def _update(self, mutations: dict):
         """
         Return oplog
         """
         data = dict(self)
-        doc, oplog = operators.update_operations(data, ops)
-        row = self._collection.update_id(
-            _id=self._id, doc=doc, replace=True, _as_document=False)
+        doc, oplog = dict_mutator.mutate(mutations, data)
+        row = self._collection.update_id(_id=self._id, doc=doc, replace=True, _as_document=False)
         self._load(row)
         return oplog
 
@@ -391,6 +408,11 @@ class CollectionItem(__Item):
         """
         self._clear_self()
         row = _parse_row(row)
+
+        self._subcollections = {}
+        if "/subcollections" in row:
+            self._subcollections = row.pop("/subcollections") or {}
+
         self._id = row.get("_id")
         super().__init__(row)
 
@@ -507,7 +529,7 @@ class Collection(object):
         """
         return list(self.find(*a, **kw))
 
-    def find(self, filters: dict = {}, sorts: dict = {}, limit: int = 10, skip: int = 0, _as_document=True) -> cursor.Cursor:
+    def find(self, filters: dict = {}, sorts: dict = {}, limit: int = 10, skip: int = 0, _as_document=True) -> dict_query.Cursor:
         """
         To fetch data from the collection
 
@@ -522,7 +544,7 @@ class Collection(object):
           skit:int - 
 
         Returns:
-          cursor.Cursor
+          dictquery.Cursor
         """
 
         # SMART QUERY
@@ -557,7 +579,7 @@ class Collection(object):
             for chunked in self.db.fetchmany(query, xparams, chunk):
                 if chunked:
                     rows = [_parse_row(row) for row in chunked]
-                    for r in _query_dict(rows, smart_filters["JSON_FILTERS"]):
+                    for r in dict_query.query(rows, smart_filters["JSON_FILTERS"]):
                         data.append(r)
                 else:
                     break
@@ -565,7 +587,7 @@ class Collection(object):
                 if _as_document:
                     data = [CollectionItem(self, d) for d in data]
             sorts = _parse_sort_dict(sorts, False)
-            return cursor.Cursor(data, sort=sorts, limit=limit, skip=skip)
+            return dict_query.Cursor(data, sort=sorts, limit=limit, skip=skip)
 
         # Skip JSON SEARCH, use only SQL.
         # No need to look into the JSON. The DB is enough
@@ -587,7 +609,7 @@ class Collection(object):
                 data = [CollectionItem(self, row) for row in res]
             else:
                 data = list(res)
-            return cursor.Cursor(data)
+            return dict_query.Cursor(data)
 
     def add(self, data: dict, _as_document: bool = True) -> CollectionItem:
         """
@@ -641,10 +663,10 @@ class Collection(object):
         _columns = None
         xparams = []
         for d in data:
+            datax, _ = dict_mutator.mutate(d)
             _id = lib.gen_id()
             ts = lib.get_timestamp()
             f_ts = ts.format(SQL_DATETIME_FORMAT)
-            datax, _ = operators.update_operations({}, d)
             datax.update({
                 "_id": _id,
                 "_created_at": ts,
@@ -736,7 +758,7 @@ class Collection(object):
             if isinstance(stmt, types.BaseType):
                 cols_stmt.append(stmt)
             elif isinstance(stmt, str):
-                cols_stmt.append(types.stmt_to_fieldtype(stmt))
+                cols_stmt.append(types.stmt_to_custom_type(stmt))
 
         self.db.add_columns(table=self.name, cols_stmt=cols_stmt)
 
@@ -758,19 +780,20 @@ class Collection(object):
 class SubCollection(object):
     _data = []
     _constraints = []
+    _item = None
+    _name = None 
 
     def __init__(self, item: CollectionItem, name: str, constraints:list=None):
         self._item = item
         self._name = name
-        self._collection_name = "/collections.%s" % name
         self._constraints = constraints
         self._load()
 
     def _load(self):
-        self._data = self._item.get(self._collection_name) or []
+        self._data = self._item._subcollections.get(self._name) or []
 
     def _commit(self):
-        self._item.set(self._collection_name, self._data)
+        self._item._set_subcollection(self._name, self._data)
 
     def _save(self, _id, data):
         _data = self._normalize_data()
@@ -787,11 +810,27 @@ class SubCollection(object):
     def __len__(self):
         return len(self._data)
 
-    def drop_subcollection(self):
-        self._item.unset(self._collection_name)
+    def __iter__(self):
+        return iter(self.find())
+
+    @property
+    def items(self):
+        """ 
+        Returns an iterator of all documents
+
+        Returns:
+            Iterator
+        """
+        return self.find()
 
     def add(self, data: dict):
-        data, _ = operators.update_operations({}, data.copy())
+        """
+        Add document
+
+        Args:
+            data:dict
+        """
+        data, _ = dict_mutator.mutate(data.copy())
 
         if self._constraints:
             for c in  self._constraints:
@@ -799,7 +838,6 @@ class SubCollection(object):
                     if self.find_one({c: data[c]}):
                         raise exceptions.ConstraintError("Key: %s" % c)
 
-
         _id = lib.gen_id()
         ts = lib.get_timestamp()
         doc = {
@@ -812,7 +850,14 @@ class SubCollection(object):
         self._commit()
         return SubCollectionItem(self, doc)
 
-    def update(self, filters, data_ops: dict):
+    def update(self, filters:dict, mutations: dict):
+        """
+        Update by filter
+
+        Args:
+            filter:dict - filter document criterai
+            mutations:dict - changes on the found documents
+        """
         _data = self._normalize_data()
         for item in self.find(filters):
             ts = lib.get_timestamp()
@@ -822,43 +867,89 @@ class SubCollection(object):
                 "_created_at": ts,
                 "_modified_at": ts
             }
-            upd, _ = operators.update_operations(item, data_ops)
+            upd, _ = dict_mutator.mutate(mutations, item)
             _data[_id] = {**upd, **_default}
         self._data = self._denormalize_data(_data)
         self._commit()
 
     def delete(self, filters: dict):
+        """
+        Delete documents based on filters
+
+        Args:
+            filters:dict
+        """
         _data = self._normalize_data()
         for item in self.find(filters):
             del _data[item.get("_id")]
         self._data = self._denormalize_data(_data)
         self._commit()
 
-    def find(self, filters: dict = {}, sorts: dict = {}, limit: int = 10, skip: int = 0) -> cursor.Cursor:
+    def filter(self, filters: dict = {}) -> dict_query.Cursor:
         """
+        Alias to find() but makes it seems fluenty
+        
+        Returns:
+            dict_query:Cursor
+        """
+        data = dict_query.query(data=self._data, filters=filters)
+        return dict_query.Cursor([SubCollectionItem(self, d) for d in data])
+
+    def find_one(self, filters:dict={}):
+        """
+        Return only one item by criteria
+
+        Return:
+            dict
+        """
+        res = self.find(filters=filters, limit=1)
+        if res:
+            return list(res)[0]
+        return None 
+
+    def get(self, _id:str):
+        """
+        Return a document from subcollection by id 
+
+        Returns:
+        """
+        return self.find_one({"_id": _id})
+
+    def find(self, filters: dict = {}, sorts: dict = {}, limit: int = 10, skip: int = 0) -> dict_query.Cursor:
+        """
+        Perform a query
+
         Args:
             filters:
             sorts:
             limit:
             skip:
         """
-        data = _query_dict(data=self._data, filters=filters)
+        data = dict_query.query(data=self._data, filters=filters)
         data = [SubCollectionItem(self, d) for d in data]
         sorts = _parse_sort_dict(sorts, False)
-        return cursor.Cursor(data, sort=sorts, limit=limit, skip=skip)
+        return dict_query.Cursor(data, sort=sorts, limit=limit, skip=skip)
 
 
 class SubCollectionItem(__Item):
+    _id = None 
 
     def __init__(self, subCollection: SubCollection, data):
-        self.subCollection = subCollection
-        self.NAMESPACE = self.subCollection._collection_name
+        self._subcollection = subCollection
         self._load(data)
 
-    def _update(self, ops):
+    @property
+    def parent(self):
+        """
+        Holds parent data
+        """
+        return self._subcollection._item
+
+    def _update(self, mutations):
         data = dict(self)
-        doc, oplog = operators.update_operations(data, ops)
-        self.subCollection._save(self._id, doc)
+        mutations = copy.deepcopy(mutations)
+        doc, oplog = dict_mutator.mutate(mutations, data)
+        self._subcollection._save(self._id, doc)
         self._load(doc)
         return oplog
 
@@ -866,6 +957,8 @@ class SubCollectionItem(__Item):
         self._id = data.get("_id")
         super().__init__(data)
 
+
+
 class _CollectionInfo__(object):
     COLLECTION_NAME = "__collections_info__"
 
@@ -915,14 +1008,14 @@ def _parse_smart_filtering(filters: dict, indexed_columns: list = []) -> dict:
     sql_filters = []
     json_filters = {}
     for k, v in filters.items():
-        if dictquery.FILTER_OPERATOR in k:
-            f, op = k.split(dictquery.FILTER_OPERATOR)
-            if f in indexed_columns and op in dictquery.SQL_OPERATORS:
-                sql_filters.append((f, dictquery.SQL_OPERATORS[op], v))
+        if dict_query.FILTER_OPERATOR in k:
+            f, op = k.split(dict_query.FILTER_OPERATOR)
+            if f in indexed_columns and op in dict_query.SQL_OPERATORS:
+                sql_filters.append((f, dict_query.SQL_OPERATORS[op], v))
                 continue
         else:
             if k in indexed_columns:
-                sql_filters.append((k, dictquery.SQL_OPERATORS["eq"], v))
+                sql_filters.append((k, dict_query.SQL_OPERATORS["eq"], v))
                 continue
         json_filters[k] = v
 
@@ -962,9 +1055,3 @@ def _ascdesc(v, as_str=True):
 
 def _parse_sort_dict(sorts: dict, as_str=True):
     return [(k, _ascdesc(v, as_str)) for k, v in sorts.items()]
-
-
-def _query_dict(data:list, filters:dict) -> List[dict]:
-    if not filters:
-        return data
-    return dictquery.DictQuery(data).execute(filters)
\ No newline at end of file
