@@ -90,20 +90,19 @@ class NumericType(BaseType):
     field_type = "Numeric"
 
 
-class FieldType(BaseType):
+class CustomType(BaseType):
     def __init__(self, field_type, *a, **kw):
         super().__init__(*a, **kw)
         self.field_type = field_type
 
 
-def stmt_to_fieldtype(stmt):
+def stmt_to_custom_type(stmt):
     """
       "column", # column only. Type is in
       "column:type", # column and type
       "column:type@index", # column, type and index
       "column:type@unique" # column type and unique index
       "column@unique" # column and unique index. Type is inferred
-
     """
     _type = "VARCHAR(255)"
     _name = stmt
@@ -118,4 +117,4 @@ def stmt_to_fieldtype(stmt):
     if ":" in _name:
         _name, _type = _name.split(":")
 
-    return FieldType(field_type=_type, name=_name, index=_index, unique=_unique)
+    return CustomType(field_type=_type, name=_name, index=_index, unique=_unique)
